@@ -87,19 +87,18 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
            or obj.getRole() in [pyatspi.ROLE_TOOL_TIP, pyatspi.ROLE_STATUS_BAR]:
             return result
 
-        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
-                               pyatspi.ROLE_DOCUMENT_WEB,
+        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_WEB,
                                pyatspi.ROLE_EMBEDDED,
                                pyatspi.ROLE_INTERNAL_FRAME,
                                pyatspi.ROLE_MATH,
-                               pyatspi.ROLE_MENU_BAR,
-                               pyatspi.ROLE_TOOL_BAR]
+                               pyatspi.ROLE_MENU_BAR]
         args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
                              pyatspi.ROLE_HEADING,
                              pyatspi.ROLE_LABEL,
                              pyatspi.ROLE_LINK,
                              pyatspi.ROLE_LIST_ITEM,
                              pyatspi.ROLE_TEXT]
+        args['stopAfterRoles'] = [pyatspi.ROLE_TOOL_BAR]
 
         if self._script.utilities.isEditableDescendantOfComboBox(obj):
             args['skipRoles'].append(pyatspi.ROLE_COMBO_BOX)
@@ -317,15 +316,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
            and not self._script.utilities.isContentSuggestion(obj):
             return []
 
-        if self._script.inSayAll() and obj == args.get('priorObj'):
+        priorObj = args.get("priorObj")
+        if obj == priorObj:
             return []
 
-        if self._script.utilities.isContentEditableWithEmbeddedObjects(obj):
+        if priorObj and priorObj in self._script.utilities.labelsForObject(obj):
+            return []
+
+        if self._script.utilities.isContentEditableWithEmbeddedObjects(obj) \
+           or self._script.utilities.isDocument(obj):
             lastKey, mods = self._script.utilities.lastKeyAndModifiers()
             if lastKey in ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
                 return []
 
-        priorObj = args.get("priorObj")
         if priorObj and priorObj.getRole() == pyatspi.ROLE_PAGE_TAB and priorObj.name == obj.name:
             return []
 
@@ -341,6 +344,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result.extend(self.voice(speech_generator.DEFAULT))
             return result
 
+        if obj.getRole() == pyatspi.ROLE_CHECK_BOX:
+            gridCell = pyatspi.findAncestor(obj, self._script.utilities.isGridCell)
+            if gridCell:
+                return super()._generateLabelOrName(gridCell, **args)
+
         return super()._generateLabelOrName(obj, **args)
 
     def _generateName(self, obj, **args):
@@ -353,6 +361,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
            and not args.get('inFlatReview'):
             return []
 
+        if self._script.utilities.hasVisibleCaption(obj):
+            return []
+
+        if self._script.utilities.isFigure(obj) and args.get('ancestorOf'):
+            caption = args.get('ancestorOf')
+            if caption.getRole() != pyatspi.ROLE_CAPTION:
+                isCaption = lambda x: x and x.getRole() == pyatspi.ROLE_CAPTION
+                caption = pyatspi.findAncestor(caption, isCaption)
+            if caption and hash(obj) in self._script.utilities.labelTargets(caption):
+                return []
+
         role = args.get('role', obj.getRole())
 
         # TODO - JD: Once the formatting strings are vastly cleaned up
@@ -474,7 +493,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateRoleName(obj, **args)
 
-        if self._script.inSayAll() and obj == args.get('priorObj'):
+        if obj == args.get('priorObj'):
             return []
 
         result = []
@@ -671,6 +690,10 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.isEditableComboBox(obj):
             return []
 
+        if args.get('formatType') not in ['basicWhereAmI', 'detailedWhereAmI']:
+            if args.get('priorObj') == obj:
+                return []
+
         position = self._script.utilities.getPositionInSet(obj)
         total = self._script.utilities.getSetSize(obj)
         if position is None or total is None:
@@ -690,6 +713,12 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         result.extend(self.voice(speech_generator.SYSTEM))
         return result
 
+    def _generateUnselectedCell(self, obj, **args):
+        if not self._script.inFocusMode():
+            return []
+
+        return super()._generateUnselectedCell(obj, **args)
+
     def _generateRealTableCell(self, obj, **args):
         result = super()._generateRealTableCell(obj, **args)
         if not self._script.inFocusMode():
@@ -746,16 +775,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             oldRole = self._overrideRole('default', args)
         elif self._script.utilities.isLink(obj):
             oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
+        elif self._script.utilities.isCustomImage(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_IMAGE, args)
         elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
             oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
         else:
             oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
 
         if not 'priorObj' in args:
-            args['priorObj'] = self._script.utilities.getPriorContext()[0]
-
-        if self._script.utilities.isLabellingContents(obj):
-            result = list(filter(lambda x: x, self.generateContext(obj, **args)))
+            document = self._script.utilities.getTopLevelDocumentForObject(obj)
+            args['priorObj'] = self._script.utilities.getPriorContext(document)[0]
 
         if not result:
             result = list(filter(lambda x: x, super().generateSpeech(obj, **args)))
@@ -789,7 +818,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 args['priorObj'] = obj
 
         if not result:
-            if self._script.inSayAll():
+            if self._script.inSayAll(treatInterruptedAsIn=False) \
+               or not _settingsManager.getSetting('speakBlankLines'):
                 string = ""
             else:
                 string = messages.BLANK
